/**************
 * leg4.c
 **************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "leg4utils.h"
#include "leg4.h"
#include "leg4asm.h"

void doAdd() ;
void doMove() ;
void doInput() ;
void doOutput() ;
void doJumpNotCarry() ;
void doJump() ;
void doNop() ;

CpuArcType gCpuArc[] = {
{ "ADD", &doAdd },
{ "MOV", &doMove },
{ "IN",  &doInput },
{ "OUT", &doOutput },
{ "JNC", &doJumpNotCarry },
{ "JMP", &doJump },
{ "NOP", &doNop },
{ 0, 0 },
} ;

FuncPtrType getFunc(char *cmd)
{
	CpuArcType *arc = &gCpuArc[0] ;	

	while((arc->mnemonic != (char*)0) && strcmp(arc->mnemonic, cmd)) {
		arc++ ;
	}

	return(arc->func) ;
}

void doAdd()
{
	char reg[32] ;
	char svalue[32] ;
	char *ptr ;
	fscanf(ifp, "%s", reg) ;
	if (ptr = strchr(reg, ',')) {
		*ptr++ = (char)0 ;
		strcpy(svalue, ptr) ;
		if (*ptr == (char)0) {
			fscanf(ifp, "%s", svalue) ;
		}
	} else {
		skipTo(ifp, ',') ;
		fscanf(ifp, "%s", svalue) ;
	}
	str2upper(reg) ;
	if (reg[0] == 'A') {
		fprintf(ofp, "0%X ;", b2i(svalue)) ;
	} else if (reg[0] == 'B') {
		fprintf(ofp, "5%X ;", b2i(svalue)) ;
	} else {
		fprintf(stderr, "Syntax Error [ ADD %s %s ]\n", reg, svalue) ;
		exit( 4 ) ;
	}
}

void doMove() 
{
	char reg[32] ;
	char str[32] ;
	char *ptr ;
	fscanf(ifp, "%s", reg) ;
	if (ptr = strchr(reg, ',')) {
		*ptr++ = (char)0 ;
		strcpy(str, ptr) ;
		if (*ptr == (char)0) {
			fscanf(ifp, "%s", str) ;
		}
	} else {
		skipTo(ifp, ',') ;
		fscanf(ifp, "%s", str) ;
	}
	str2upper(reg) ;
	str2upper(str) ;
	if (reg[0] == 'A') { /* MOV A */
		if (str[0] == 'B') { /* MOV A, B */
			fprintf(ofp, "10 ;") ;
		} else { /* MOV A, xxxx */
			fprintf(ofp, "3%X ;", b2i(str)) ;
		}
	} else if (reg[0] == 'B') { /* MOV B */
		if (str[0] == 'A') { /* MOV B, A */
			fprintf(ofp, "40 ;") ;
		} else { /* MOV B, xxxx */
			fprintf(ofp, "7%X ;", b2i(str)) ;
		}
	} else {
		fprintf(stderr, "Syntax Error [ MOV %s %s ]\n", reg, str) ;
		exit( 5 ) ;
	}
}

void doJumpNotCarry()
{
	char str[32] ;
	fscanf(ifp, "%s", str) ;

	fprintf(ofp, "E%X ;", b2i(str)) ;
}

void doJump()
{
	char str[32] ;
	fscanf(ifp, "%s", str) ;

	fprintf(ofp, "F%X ;", b2i(str)) ;
}

void doInput()
{
	char reg[32] ;
	fscanf(ifp, "%s", reg) ;
	str2upper(reg) ;
	if (reg[0] == 'A') {
		fprintf(ofp, "20 ;") ;
	} else if (reg[0] == 'B') {
		fprintf(ofp, "60 ;") ;
	} else {
		fprintf(stderr, "Unknown register %s\n", reg) ;
		exit( 7 ) ;	
	}
}

void doOutput()
{
	char str[32] ;
	fscanf(ifp, "%s", str) ;
	str2upper(str) ;
	if (str[0] == 'B') { /* OUT B */
		fprintf(ofp, "90 ;") ;
	} else if ((str[0] == '0')||(str[0] == '1')) { /* OUT xxxx */
		fprintf(ofp, "B%X ;", b2i(str)) ;
	} else {
		fprintf(stderr, "Syntax Error [ OUT %s ]\n", str) ;
		exit( 8 ) ;
	}
}

void doNop()
{
	fprintf(ofp, "00 ;") ;
}
