/*****************
 * leg4asmv.c
 * System Verilog Version
 ****************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "leg4utils.h"
#include "leg4.h"
#include "leg4asm.h"

char *header[] = {
	"module leg4_rom (\n",
	"\tinput  logic [3:0] address,\n",
	"\toutput logic [7:0] out\n",
	") ;\n",
	"\tfunction [7:0] rom ;\n",
	"\t\tinput [3:0] address ;\n",
	"\t\tbegin\n",
	"\t\t\tcase (address)\n",
	0
} ;

char *tailer[] = {
	"\t\t\tdefault: rom = 8'h00 ;\n",
	"\t\t\tendcase\n",
	"\t\tend\n",
	"\tendfunction : rom\n",
	"\n",
	"\tassign out = rom(address) ;\n",
	"\n",
	"endmodule : leg4_rom\n",
	0
} ;

FILE *ifp ;
FILE *ofp ;
int gAddress = 0 ;
int gLineNo = 0 ;

int main(int argc, char *argv[])
{
	getOpt(argc, argv) ;
	doIt() ;
	cleanUp() ;
	return(0) ;
}

void getOpt(int argc, char *argv[])
{
    if ((argc > 1)&&(strcmp(argv[1], "-"))) {
        ifp = efopen(argv[1], "r") ;
    } else {
        ifp = stdin ;
    }
    if ((argc > 2)&&(strcmp(argv[2], "-"))) {
        ofp = efopen(argv[2], "w") ;
    } else {
        ofp = stdout ;
    }
}

void cleanUp() 
{
	fclose(ifp) ;
	fclose(ofp) ;
}

void writeAddress()
{
	fprintf(ofp, "\t\t\t4'h%01X: rom = 8'h", gAddress ) ;
}

int incAddress()
{
	gAddress++ ;
	return( gAddress ) ;
}

void writeVerilogHeader(FILE *ofp)
{
	char **ptr = header ;
	while(ptr && *ptr) {
		fprintf(ofp, *ptr++) ;
	}
}

void writeVerilogTailer(FILE *ofp)
{	
	char **ptr = tailer ;
	int adr ;
	for (adr = gAddress ; adr < 0x10 ; adr++ ) {
		fprintf(ofp, "\t\t\t4'h%01X: rom = 8'h00 ;\n", adr) ;
	}
	while(ptr && *ptr) {
		fprintf(ofp, *ptr++) ;
	}
}

void doIt()
{
	char command[256] ;
	FuncPtrType func ;

	writeVerilogHeader(ofp) ;

	while(fscanf(ifp, "%s", command) != EOF) {
		gLineNo++ ;
		if (*command != '!') {
			str2upper(command) ;
			func = getFunc(command) ;
			if (func != 0) {
				writeAddress() ;
				func() ;
				incAddress() ;
			} else {
				fprintf(stderr, "Syntax Error at %d : %s\n", gLineNo, command) ;
				exit( 9 ) ;
			}
			skipToEOL(ifp) ;	
		} else { /* take care of comment line */
			fprintf(ofp, "// ") ;
			dupToEOL(ifp, ofp) ;
		}
		fprintf(ofp, "\n") ;
	}

	writeVerilogTailer(ofp) ;
}
