module test_pwm_servo(
    input              clk,
    input              nrst,
    input        [3:0] sw,
    output logic       pulse,
    output logic [7:0] led
) ;

    parameter MAX100KHZ = 26'd269  ; // (27MHz / 100kHz) - 1
    parameter PWM50HZ   = 16'd1999 ; // (100kHz / 50Hz) - 1
    parameter PWM_MIN   =   16'd60 ;
    parameter PWM_POS2  =  16'd100 ;
    parameter PWM_POS3  =  16'd160 ;
    parameter PWM_MAX   =  16'd230 ;

    logic        clk100khz ;
    logic [15:0] compare ;

    assign led  = { sw[3], sw[3], sw[2], sw[2], sw[1], sw[1], sw[0], sw[0] } ;

    clkdiv clkdiv_1 (.*, .max(MAX100KHZ), .clk_out(clk100khz), .tc()) ;

    pwm pwm_1 (.clk(clk100khz), .nrst(nrst), .period(PWM50HZ), .compare(compare), .line(pulse), .tc()) ;

    always @(negedge nrst or posedge clk) begin
        if (nrst == 1'b0) 
            compare <= 'd100 ;
        else begin
            case (sw)
            4'b0111: compare <= PWM_MIN  ;
            4'b1011: compare <= PWM_POS2 ;
            4'b1101: compare <= PWM_POS3 ;
            4'b1110: compare <= PWM_MAX  ;
            default: compare <= compare  ;
            endcase
        end
    end /* always */

endmodule : test_pwm_servo
