module stopwatch (
    input              nrst, 
    input              clk, 
    input              btn,
    output logic [7:0] seg,
    output logic [3:0] dig,
    output logic [7:0] led ) ;

    parameter MAX100HZ = 26'd269999 ;
    parameter MAX10HZ  = 26'd2699999 ;
    parameter MAX160HZ = 26'd168749 ;
    parameter MAX400HZ = 26'd67499 ;
    parameter NINE     = 4'd9 ;
    parameter FIVE     = 4'd5 ;
    parameter EIGHT    = 4'd8 ;

    logic       psw, en, nen, dot ;
    logic       clk100hz, clk10hz, clk1s, clk10s, clk1m ; /* clocks */
    logic [3:0] cnt100hz, cnt10hz, cnt1s, cnt10s, cnt1m ; /* counts */
    logic       clk400hz   ; /* clock for debounce */

    logic [3:0][7:0] seg_in ;

    assign nen = ~en ;

    assign dot = (nen | ((cnt10hz < 5) ? 1'b1 : 1'b0)) ;

    clkdiv clkdiv_1       (.*, .max(MAX100HZ), .clk_out(),          .tc(clk100hz)) ;

    clkdiv clkdiv_2       (.*, .max(MAX400HZ), .clk_out(),          .tc(clk400hz)) ;
 
    clkdiv clkdiv_3       (.*, .max(MAX160HZ), .clk_out(clk160hz),  .tc()) ;

    debounce debounce_1   (.*, .clk(clk400hz), .key_in(btn), .key_out(psw)) ;

    toggle toggle_2       (.*, .in(psw), .out(en)) ;

    cntr4maxe cntr4maxe_1 (.*, .up(clk100hz), .max(NINE),  .cnt(cnt100hz), .tc(clk10hz)) ;

    cntr4max  cntr4max_1  (.*, .up(clk10hz),  .max(NINE),  .cnt(cnt10hz),  .tc(clk1s)) ;

    cntr4max  cntr4max_2  (.*, .up(clk1s),    .max(NINE),  .cnt(cnt1s),    .tc(clk10s)) ;

    cntr4max  cntr4max_3  (.*, .up(clk10s),   .max(FIVE),  .cnt(cnt10s),   .tc(clk1m)) ;

    cntr4max  cntr4max_4  (.*, .up(clk1m),    .max(EIGHT), .cnt(cnt1m),    .tc()) ;

    drv7seg  drv7seg_1 (.in(cnt100hz), .dp(1'b0), .seg(seg_in[0])) ;
    drv7seg  drv7seg_2 (.in(cnt10hz),  .dp(1'b0), .seg(seg_in[1])) ;
    drv7seg  drv7seg_3 (.in(cnt1s),    .dp(dot),  .seg(seg_in[2])) ;
    drv7seg  drv7seg_4 (.in(cnt10s),   .dp(1'b0), .seg(seg_in[3])) ;
    dec_led  dec_led_1 (.in(cnt1m),    .led(led)) ;

    mux7seg  inst20 (.*, .clk(clk160hz)) ;

endmodule /* stopwatch) */