module stump (
    input        [1:0] ob_sw,  // On board switch      (Negative Logic)
    input        [7:0] tsw,    // toggle switch        (Negative Logic)
    input        [3:0] psw,    // push switch          (Negative Logic)
    output logic [5:0] ob_led, // On board led         (Negative Logic)
    output logic [7:0] led,    // LED                  (Negative Logic)
    output logic [7:0] seg,    // SEG a,b,c,d,e,f,g,dp (Positive Logic)
    output logic [3:0] dig     // Digit select         (Negative Logic)
) ;

    assign led[7:4]    =  tsw[7:4] ;
    assign led[3:0]    = ~tsw[3:0] ;
    assign dig         =  tsw[7:4] ;
    assign ob_led[3:0] =  psw[3:0] ;
    assign ob_led[5]   =  ob_sw[1] ;
    assign ob_led[4]   =  ob_sw[0] ;

    drv7seg drv7seg_1 (.*, .value(tsw[3:0]), .dp(psw[0])) ;

endmodule : stump

module drv7seg (
    input  [3:0] value,
    input        dp,
    output [7:0] seg
) ;

    function  [6:0] f ;
        input [3:0] in ;
        begin
            case (in)
            4'h0: f = 7'h7E ;
            4'h1: f = 7'h30 ;
            4'h2: f = 7'h6D ;
            4'h3: f = 7'h79 ;
            4'h4: f = 7'h33 ;
            4'h5: f = 7'h5B ;
            4'h6: f = 7'h5F ;
            4'h7: f = 7'h70 ;
            4'h8: f = 7'h7F ;
            4'h9: f = 7'h7B ;
            4'hA: f = 7'h77 ;
            4'hB: f = 7'h1F ;
            4'hC: f = 7'h4E ;
            4'hD: f = 7'h3D ;
            4'hE: f = 7'h4F ;
            4'hF: f = 7'h47 ;
            endcase
        end
    endfunction : f

    assign seg = { f(value), ~dp } ;

endmodule : drv7seg
