module rx (
    input              clk2x,
    input              nrst,
    input              rxd,
    output logic [7:0] data,
    output logic       ready
) ;

    logic [1:0]        state    ;
    logic [7:0]        work     ;
    logic [7:0]        mask     ;
    logic [4:0]        count    ;
    logic              prev_rxd ;

    always_ff @(negedge nrst or posedge clk2x) begin
        if (nrst == 1'b0) begin
            state    <= 2'd0  ; /* idle */
            data     <= 8'h00 ; 
            work     <= 8'h00 ;
            ready    <= 1'd0  ;
            prev_rxd <= 1'b1  ;
            mask     <= 8'h0  ;
        end else begin
            case (state)
            2'd0: begin /* idle */
                if ({rxd, prev_rxd} == 2'b01) begin /* start bit received */
                    work  <= 8'h00 ;
                    mask  <= 8'h01 ;
                    ready <= 1'b0  ;
                    count <= 5'd0  ;
                    state <= 2'd1  ; /* go to receiving */
                end
            end
            2'd1: begin /* receiving */
                if (count[0] == 1'b1) begin
                    if (mask == 8'd0) begin /* receive complete */
                        count <= 5'd0 ;
                        state <= 2'd2 ; /* go to wait for stop */
                    end else if (rxd == 1'b1) begin
                        work  <= work | mask ;
                    end
                    mask <= mask << 3'd1 ;
                end
                count <= count + 5'd1 ;
            end
            2'd2: begin /* wait for stop condition */
                if (count[0] == 1'b1) begin
                    if (rxd == 1'b1) begin
                        data  <= work ;
                        ready <= 1'b1 ;
                        state <= 2'd0 ; /* go to idle */
                    end
                end else if (count >= 5'd20) begin /* break received */
                    ready  <= 1'b0 ;
                    state <= 2'd0 ;
                end
                count <= count + 5'd1 ;
            end
            2'd3: begin /* unexpected state 3 go to idle */
                state <= 2'd0 ;
            end
            endcase
            prev_rxd <= rxd ;
        end /* if */
    end /* always */
endmodule : rx
