module test_uart (
    input              clk, // 27MHz Clock
    input              nrst,
    input [7:0]        tx_data,
    input              send,
    input              rxd, // FPGA_RX : Pin 18
    output logic       txd, // FPGA_TX : Pin 17
    output logic [7:0] led,
    output logic [7:0] seg,
    output logic [3:0] dig
) ;

    parameter MAXBAUD   = 26'd233 ; // (27MHz / 115200) - 1 
    parameter MAXBAUDX2 = 26'd116 ; // (27MHz / 115200) - 1 
    parameter MAX400HZ  = 26'd67499  ; // (27MHz / 400) - 1
    parameter MAX160HZ  = 26'd168749 ; // (27MHz / 160) - 1

    logic            tx_busy ;
    logic            data_ready ;
    logic            tcbaudhz, tcbaudx2hz ;
    logic            clkbaudhz, clkbaudx2hz, clk400hz ;
    logic            clk160hz ;    /* clock for mux7seg */
    logic            tx_send ;
    logic [7:0]      rx_data ;
    logic [3:0][7:0] seg_in ;

    assign led[7]   = rxd ;
    assign led[6]   = data_ready ;
    assign led[5:4] = 2'd0 ;
    assign led[3]   = txd ;
    assign led[2]   = tx_busy ;
    assign led[1:0] = 2'd0 ;

    clkdiv clkdiv_1 (.*, .max(MAXBAUD),   .clk_out(clkbaudhz),   .tc()) ; // for tx
    clkdiv clkdiv_2 (.*, .max(MAXBAUDX2), .clk_out(clkbaudx2hz), .tc()) ; // for rx
    clkdiv clkdiv_3 (.*, .max(MAX400HZ),  .clk_out(clk400hz),    .tc()) ; // for debounce
    clkdiv clkdiv_4 (.*, .max(MAX160HZ),  .clk_out(clk160hz),    .tc()) ; // for mux7seg

    debounce debounce_1 (.clk(clk400hz), .nrst(nrst), .key_in(send), .key_out(tx_send)) ;

    tx tx_1 (.clk(clkbaudhz), .nrst(nrst), .send(tx_send), .in(tx_data), .txd(txd), .busy(tx_busy)) ;

    rx rx_1 (.clk2x(clkbaudx2hz), .nrst(nrst), .rxd(rxd), .data(rx_data), .ready(data_ready)) ;

    drv7seg drv7seg_1 (.in(rx_data[7:4]), .dp(1'b0), .seg(seg_in[3])) ;
    drv7seg drv7seg_2 (.in(rx_data[3:0]), .dp(1'b0), .seg(seg_in[2])) ;
    drv7seg drv7seg_3 (.in(tx_data[7:4]), .dp(1'b0), .seg(seg_in[1])) ;
    drv7seg drv7seg_4 (.in(tx_data[3:0]), .dp(1'b0), .seg(seg_in[0])) ;

    mux7seg mux7seg_1 (.*, .clk(clk160hz)) ;

endmodule : test_uart
