module tx (
    input         clk,
    input         nrst,
    input         send,
    input [7:0]   in,
    output logic  txd,
    output logic  busy
) ;
    logic [1:0] state     ;
    logic [7:0] buffer    ;
    logic [7:0] mask      ;
    logic       prev_send ;

    always_ff @(negedge nrst or posedge clk) begin
        if (nrst == 1'b0) begin
            state     <= 2'd0 ;
            buffer    <= 8'd1 ;
            busy      <= 1'b0 ;
            txd       <= 1'b1 ;
            prev_send <= 1'b0 ;
        end else begin
            case (state)
            2'd0: /* idle */
                if ({send, prev_send} == 2'b10) begin /* rising edge of send */
                    buffer <= in ;
                    txd    <= 1'b0 ; /* start bit */
                    mask   <= 8'd1 ;
                    state  <= 2'd1 ; /* change to send */
                    busy   <= 1'b1 ; 
                end else begin
                    txd    <= 1'b1 ;
                end
            2'd1: /* sending data */
                if (mask != 8'd0) begin
                    txd   <= (mask & buffer) ? 1'b1 : 1'b0 ;
                    mask  <= mask << 1 ;
                end else begin
                    txd   <= 1'b1 ;
                    state <= 2'd2 ;
                end
            2'd2: /* data was sent returning to idle */
                begin
                    txd   <= 1'b1 ;
                    busy  <= 1'b0 ;
                    state <= 2'd0 ;
                end
            2'd3: /* unexpected state, goto idle */
                begin
                    txd   <= 1'b1 ;
                    busy  <= 1'b0 ;
                    state <= 2'd0 ;
                end
            endcase
            prev_send <= send ;
        end /* if */
    end /* always */
endmodule : tx 